import 'UtilsConversion.dart';
import 'Unit.dart';

//Available POWER units
enum POWER {
  watt,
  milliwatt,
  kilowatt,
  megawatt,
  gigawatt,
  european_horse_power,
  imperial_horse_power,
}

class Power {
  //Map between units and its symbol
  final Map<POWER, String> mapSymbols = {
    POWER.watt: 'W',
    POWER.milliwatt: 'mW',
    POWER.kilowatt: 'kW',
    POWER.megawatt: 'MW',
    POWER.gigawatt: 'GW',
    POWER.european_horse_power: 'hp(M)',
    POWER.imperial_horse_power: 'hp(I)',
  };

  int significantFigures;
  bool removeTrailingZeros;
  List<Unit> unitList = [];
  Node _unit_conversion;

  ///Class for power conversions, e.g. if you want to convert 1 kilowatt in european horse power:
  ///```dart
  ///var power = Power(removeTrailingZeros: false);
  ///power.Convert(Unit(POWER.kilowatt, value: 1));
  ///print(POWER.european_horse_power);
  /// ```
  Power({int significantFigures = 10, bool removeTrailingZeros = true}) {
    this.significantFigures = significantFigures;
    this.removeTrailingZeros = removeTrailingZeros;
    POWER.values.forEach((element) => unitList.add(Unit(element, symbol: mapSymbols[element])));
    _unit_conversion = Node(name: POWER.watt, leafNodes: [
      Node(
        coefficientProduct: 1e-3,
        name: POWER.milliwatt,
      ),
      Node(
        coefficientProduct: 1e3,
        name: POWER.kilowatt,
      ),
      Node(
        coefficientProduct: 1e6,
        name: POWER.megawatt,
      ),
      Node(
        coefficientProduct: 1e9,
        name: POWER.gigawatt,
      ),
      Node(
        coefficientProduct: 735.49875,
        name: POWER.european_horse_power,
      ),
      Node(
        coefficientProduct: 745.69987158,
        name: POWER.imperial_horse_power,
      ),
    ]);
  }

  ///Converts a unit with a specific name (e.g. POWER.european_horse_power) and value to all other units
  void Convert(POWER name, double value) {
    _unit_conversion.clearAllValues();
    _unit_conversion.clearSelectedNode();
    var currentUnit = _unit_conversion.getByName(name);
    currentUnit.value = value;
    currentUnit.selectedNode = true;
    currentUnit.convertedNode = true;
    _unit_conversion.convert();
    for (var i = 0; i < POWER.values.length; i++) {
      unitList[i].value = _unit_conversion.getByName(POWER.values.elementAt(i)).value;
      unitList[i].stringValue = mantissaCorrection(unitList[i].value, significantFigures, removeTrailingZeros);
    }
  }

  Unit get watt => _getUnit(POWER.watt);
  Unit get milliwatt => _getUnit(POWER.milliwatt);
  Unit get kilowatt => _getUnit(POWER.kilowatt);
  Unit get megawatt => _getUnit(POWER.megawatt);
  Unit get gigawatt => _getUnit(POWER.gigawatt);
  Unit get european_horse_power => _getUnit(POWER.european_horse_power);
  Unit get imperial_horse_power => _getUnit(POWER.imperial_horse_power);

  ///Returns all the power units converted with prefixes
  List<Unit> getAll() {
    return unitList;
  }

  ///Returns the Unit with the corresponding name
  Unit _getUnit(var name) {
    return unitList.where((element) => element.name == name).first;
  }
}
